# This file was auto-generated by Fern from our API Definition.

from collections import ChainMap
from typing import Any, Dict, Optional

from .pydantic_utilities import pydantic_v1


# Flattens dicts to be of the form {"key[subkey][subkey2]": value} where value is not a dict
def traverse_query_dict(dict_flat: Dict[str, Any], key_prefix: Optional[str] = None) -> Dict[str, Any]:
    result = {}
    for k, v in dict_flat.items():
        key = f"{key_prefix}[{k}]" if key_prefix is not None else k
        if isinstance(v, dict):
            result.update(traverse_query_dict(v, key))
        else:
            result[key] = v
    return result


def single_query_encoder(query_key: str, query_value: Any) -> Dict[str, Any]:
    if isinstance(query_value, pydantic_v1.BaseModel) or isinstance(query_value, dict):
        if isinstance(query_value, pydantic_v1.BaseModel):
            obj_dict = query_value.dict(by_alias=True)
        else:
            obj_dict = query_value
        return traverse_query_dict(obj_dict, query_key)

    return {query_key: query_value}


def encode_query(query: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:
    return dict(ChainMap(*[single_query_encoder(k, v) for k, v in query.items()])) if query is not None else None



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.country import Country
from ...generated_types.types.relationships import Relationships
from .traversal_data import TraversalData


class TraversalResponse(pydantic_v1.BaseModel):
    """
    OK

    Examples
    --------
    from sayari import (
        EntityDetails,
        RelationshipInfo,
        RiskData,
        SourceCountInfo,
        TraversalData,
        TraversalPath,
        TraversalRelationshipData,
        TraversalResponse,
    )

    TraversalResponse(
        min_depth=1,
        max_depth=4,
        relationships=[],
        countries=[],
        types=[],
        name="",
        watchlist=False,
        psa=True,
        offset=0,
        limit=1,
        partial_results=False,
        next=True,
        explored_count=9999999,
        data=[
            TraversalData(
                source="mGq1lpuqKssNWTjIokuPeA",
                target=EntityDetails(
                    id="puXgX-Swu-Z8bCMW2_FSdQ",
                    label="MARIE LEBLANC",
                    degree=1,
                    entity_url="/v1/entity/puXgX-Swu-Z8bCMW2_FSdQ",
                    pep=False,
                    psa_count=0,
                    sanctioned=False,
                    closed=False,
                    trade_count={"sent": 0, "received": 0},
                    type="person",
                    identifiers=[],
                    addresses=[
                        "202 HAMMERSMITH ROAD UNITED KINGDOM",
                        "202 HAMMERSMITH ROAD BRITISH ISLES",
                    ],
                    countries=["USA", "GBR"],
                    relationship_count={"officer_of": 1},
                    source_count={
                        "a447a7b622c4ead6e1caf94983dc2337": SourceCountInfo(
                            count=6,
                            label="USA California Secretary of State",
                        )
                    },
                    risk={
                        "basel_aml": RiskData(
                            value=3.99,
                            metadata={"country": ["GBR"]},
                            level="relevant",
                        ),
                        "cpi_score": RiskData(
                            value=78.0,
                            metadata={"country": ["GBR"]},
                            level="relevant",
                        ),
                    },
                    user_attribute_count={},
                    user_record_count=0,
                    user_related_entities_count=0,
                    user_relationship_count={},
                    related_entities_count=1,
                    attribute_count={"name": 1, "address": 2, "country": 2},
                    reference_id="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1649116800000:0ce01cbfaa586fcfb82164f0e5a010d5",
                ),
                path=[
                    TraversalPath(
                        field="has_officer",
                        entity=EntityDetails(
                            id="puXgX-Swu-Z8bCMW2_FSdQ",
                            label="MARIE LEBLANC",
                            degree=1,
                            entity_url="/v1/entity/puXgX-Swu-Z8bCMW2_FSdQ",
                            pep=False,
                            psa_count=0,
                            sanctioned=False,
                            closed=False,
                            trade_count={"sent": 0, "received": 0},
                            type="person",
                            identifiers=[],
                            addresses=[
                                "202 HAMMERSMITH ROAD UNITED KINGDOM",
                                "202 HAMMERSMITH ROAD BRITISH ISLES",
                            ],
                            countries=["USA", "GBR"],
                            relationship_count={"officer_of": 1},
                            source_count={
                                "a447a7b622c4ead6e1caf94983dc2337": SourceCountInfo(
                                    count=6,
                                    label="USA California Secretary of State",
                                )
                            },
                            risk={
                                "basel_aml": RiskData(
                                    value=3.99,
                                    metadata={"country": ["GBR"]},
                                    level="relevant",
                                ),
                                "cpi_score": RiskData(
                                    value=78.0,
                                    metadata={"country": ["GBR"]},
                                    level="relevant",
                                ),
                            },
                            user_attribute_count={},
                            user_record_count=0,
                            user_related_entities_count=0,
                            user_relationship_count={},
                            related_entities_count=1,
                            attribute_count={"name": 1, "address": 2, "country": 2},
                            reference_id="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1649116800000:0ce01cbfaa586fcfb82164f0e5a010d5",
                        ),
                        relationships={
                            "has_officer": TraversalRelationshipData(
                                values=[
                                    RelationshipInfo(
                                        record="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1649116800000",
                                        acquisition_date="2022-04-05",
                                        attributes={
                                            "position": [
                                                {"value": "Chief Executive Officer"}
                                            ]
                                        },
                                    ),
                                    RelationshipInfo(
                                        record="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1678752000000",
                                        acquisition_date="2023-03-14",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1685664000000",
                                        acquisition_date="2023-06-02",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1693872000000",
                                        acquisition_date="2023-09-05",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1701648000000",
                                        acquisition_date="2023-12-04",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="9139b58de1bdb0157a1a1e54e56df6d3/4781466/1709769600000",
                                        acquisition_date="2024-03-07",
                                        attributes={},
                                    ),
                                ],
                                last_observed="2024-03-07",
                            )
                        },
                    )
                ],
            )
        ],
    )
    """

    min_depth: int
    max_depth: int
    relationships: typing.List[Relationships]
    countries: typing.List[Country]
    types: typing.List[str]
    name: str
    watchlist: bool
    psa: bool
    offset: int
    limit: int
    next: bool
    partial_results: bool
    data: typing.List[TraversalData]
    sanctioned: typing.Optional[bool] = None
    pep: typing.Optional[bool] = None
    explored_count: int

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.auth_response import AuthResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthResponse:
        """
        Hit the auth endpoint to get a bearer token

        Parameters
        ----------
        client_id : str

        client_secret : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthResponse
            OK

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.auth.get_token(
            client_id="your client_id here",
            client_secret="your client_secret here",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "oauth/token",
            method="POST",
            json={
                "client_id": client_id,
                "client_secret": client_secret,
                "audience": "sayari.com",
                "grant_type": "client_credentials",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(AuthResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthResponse:
        """
        Hit the auth endpoint to get a bearer token

        Parameters
        ----------
        client_id : str

        client_secret : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthResponse
            OK

        Examples
        --------
        import asyncio

        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.auth.get_token(
                client_id="your client_id here",
                client_secret="your client_secret here",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "oauth/token",
            method="POST",
            json={
                "client_id": client_id,
                "client_secret": client_secret,
                "audience": "sayari.com",
                "grant_type": "client_credentials",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(AuthResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

Entities = typing.Union[
    typing.Literal[
        "generic",
        "unknown",
        "aircraft",
        "intellectual_property",
        "person",
        "vessel",
        "legal_matter",
        "security",
        "property",
        "company",
        "tradename",
        "shipment",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from .add_attribute import AddAttribute
from .attribute_properties import AttributeProperties
from .attribute_response import AttributeResponse
from .attribute_response_data import AttributeResponseData
from .update_attribute import UpdateAttribute

__all__ = ["AddAttribute", "AttributeProperties", "AttributeResponse", "AttributeResponseData", "UpdateAttribute"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .data_source import DataSource
from .hs_code_info import HsCodeInfo
from .monetary_value import MonetaryValue
from .shipment_address import ShipmentAddress
from .shipment_identifier import ShipmentIdentifier
from .source_or_destination_entity import SourceOrDestinationEntity
from .weight import Weight


class Shipment(pydantic_v1.BaseModel):
    id: str
    type: str
    buyer: typing.List[SourceOrDestinationEntity]
    supplier: typing.List[SourceOrDestinationEntity]
    arrival_date: typing.Optional[str] = None
    departure_date: typing.Optional[str] = None
    departure_address: typing.Optional[ShipmentAddress] = None
    arrival_address: typing.Optional[ShipmentAddress] = None
    monetary_value: typing.List[MonetaryValue]
    weight: typing.List[Weight]
    identifier: typing.List[ShipmentIdentifier]
    sources: typing.List[DataSource]
    hs_codes: typing.List[HsCodeInfo]
    product_descriptions: typing.List[str]
    record: str = pydantic_v1.Field()
    """
    The unique identifier for a record in the database
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.country import Country
from ...generated_types.types.risk import Risk
from .business_purpose import BusinessPurpose


class SourceOrDestinationEntity(pydantic_v1.BaseModel):
    id: str = pydantic_v1.Field()
    """
    Unique identifier of the entity
    """

    names: typing.List[str]
    risks: typing.Dict[Risk, typing.Any] = pydantic_v1.Field()
    """
    [Risks](/sayari-library/ontology/risk-factors)
    """

    business_purpose: typing.List[BusinessPurpose] = pydantic_v1.Field()
    """
    [Business Purpose](/sayari-library/ontology/enumerated-types#business-purpose-standard)
    """

    countries: typing.List[Country] = pydantic_v1.Field()
    """
    [Countries](/sayari-library/ontology/enumerated-types#country)
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BusinessPurpose,
    BuyerSearchResponse,
    DataSource,
    HsCode,
    HsCodeInfo,
    MonetaryValue,
    Shipment,
    ShipmentAddress,
    ShipmentCountry,
    ShipmentIdentifier,
    ShipmentMetadata,
    ShipmentSearchResponse,
    SourceOrDestinationEntity,
    SupplierMetadata,
    SupplierOrBuyer,
    SupplierSearchResponse,
    TradeFilterList,
    Weight,
)

__all__ = [
    "BusinessPurpose",
    "BuyerSearchResponse",
    "DataSource",
    "HsCode",
    "HsCodeInfo",
    "MonetaryValue",
    "Shipment",
    "ShipmentAddress",
    "ShipmentCountry",
    "ShipmentIdentifier",
    "ShipmentMetadata",
    "ShipmentSearchResponse",
    "SourceOrDestinationEntity",
    "SupplierMetadata",
    "SupplierOrBuyer",
    "SupplierSearchResponse",
    "TradeFilterList",
    "Weight",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.api_error import ApiError
from ..types.rate_limit_response import RateLimitResponse


class RateLimitExceeded(ApiError):
    def __init__(self, body: RateLimitResponse):
        super().__init__(status_code=429, body=body)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.company_status import CompanyStatus
from ...generated_types.types.country import Country
from ...generated_types.types.risk import Risk
from .upstream_tiers import UpstreamTiers


class ProjectEntitiesFilter(pydantic_v1.BaseModel):
    risk: typing.Optional[typing.List[Risk]] = pydantic_v1.Field(default=None)
    """
    Filter by [risk factor](/sayari-library/ontology/risk-factors) ID.
    """

    upstream_risk: typing.Optional[typing.List[Risk]] = pydantic_v1.Field(default=None)
    """
    Filter by upstream (supply chain) [risk factor](/sayari-library/ontology/risk-factors) ID.
    """

    upstream_risk_tiers: typing.Optional[typing.List[UpstreamTiers]] = pydantic_v1.Field(default=None)
    """
    Filter by upstream (supply chain) tiers that has one or more risks
    """

    country: typing.Optional[typing.List[Country]] = pydantic_v1.Field(default=None)
    """
    Filter by [country](/sayari-library/ontology/enumerated-types#country).
    """

    upstream_country: typing.Optional[typing.List[Country]] = pydantic_v1.Field(default=None)
    """
    Filter by upstream (supply chain) [country](/sayari-library/ontology/enumerated-types#country).
    """

    upstream_country_tiers: typing.Optional[typing.List[UpstreamTiers]] = pydantic_v1.Field(default=None)
    """
    Filter by upstream (supply chain) tiers that has one or more countries
    """

    business_purpose: typing.Optional[typing.List[str]] = pydantic_v1.Field(default=None)
    """
    Filter by HS code, HS code description, or business description.
    """

    label_fuzzy: typing.Optional[typing.List[str]] = pydantic_v1.Field(alias="label.fuzzy", default=None)
    """
    Filter by entity label with fuzzy matching.
    """

    city_fuzzy: typing.Optional[typing.List[str]] = pydantic_v1.Field(alias="city.fuzzy", default=None)
    """
    Filter by entity city with fuzzy matching.
    """

    state_fuzzy: typing.Optional[typing.List[str]] = pydantic_v1.Field(alias="state.fuzzy", default=None)
    """
    Filter by entity address state with fuzzy matching.
    """

    identifier_fuzzy: typing.Optional[typing.List[str]] = pydantic_v1.Field(alias="identifier.fuzzy", default=None)
    """
    Filter by entity identifier attributes with fuzzy matching.
    """

    source_exact: typing.Optional[typing.List[str]] = pydantic_v1.Field(alias="source.exact", default=None)
    """
    Filter by entity source ID.
    """

    status_exact: typing.Optional[typing.List[CompanyStatus]] = pydantic_v1.Field(alias="status.exact", default=None)
    """
    Filter by entity [company status](/sayari-library/ontology/enumerated-types#company-status).
    """

    bounds: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Filter by a geographical bounding box. The value is a pipe-delimited set of four values representing the top, left, bottom, and right sides of the bounding box, in that order. The pipes should be URL-encoded as `%7C`. The top coordinate must greater than the bottom coordinate, and the left coordinate must be less than the right coordinate. A sample is `55.680357237879136|-71.53607290158526|41.10876347746233|-40.963927098414736`
    """

    custom_field_name: typing.Optional[typing.List[str]] = pydantic_v1.Field(alias="custom_<field name>", default=None)
    """
    <Warning>This property is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> custom user key/value pairs (key must be prefixed with "custom\_" and value must be "string" type)
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .additional_information_info import AdditionalInformationInfo
from .address_info import AddressInfo
from .business_purpose_info import BusinessPurposeInfo
from .company_type_info import CompanyTypeInfo
from .contact_info import ContactInfo
from .country_info import CountryInfo
from .date_of_birth_info import DateOfBirthInfo
from .finances_info import FinancesInfo
from .financials_info import FinancialsInfo
from .gender_info import GenderInfo
from .generic_info import GenericInfo
from .identifier_info import IdentifierInfo
from .measurement_info import MeasurementInfo
from .monetary_value_info import MonetaryValueInfo
from .name_info import NameInfo
from .person_status_info import PersonStatusInfo
from .position_info import PositionInfo
from .risk_intelligence_info import RiskIntelligenceInfo
from .shares_info import SharesInfo
from .status_info import StatusInfo
from .translated_name_info import TranslatedNameInfo
from .weak_identifier_info import WeakIdentifierInfo


class AttributeDetails(pydantic_v1.BaseModel):
    additional_information: typing.Optional[AdditionalInformationInfo] = None
    address: typing.Optional[AddressInfo] = None
    business_purpose: typing.Optional[BusinessPurposeInfo] = None
    company_type: typing.Optional[CompanyTypeInfo] = None
    contact: typing.Optional[ContactInfo] = None
    country: typing.Optional[CountryInfo] = None
    date_of_birth: typing.Optional[DateOfBirthInfo] = None
    finances: typing.Optional[FinancesInfo] = None
    financials: typing.Optional[FinancialsInfo] = None
    gender: typing.Optional[GenderInfo] = None
    generic: typing.Optional[GenericInfo] = None
    identifier: typing.Optional[IdentifierInfo] = None
    measurement: typing.Optional[MeasurementInfo] = None
    monetary_value: typing.Optional[MonetaryValueInfo] = None
    name: typing.Optional[NameInfo] = None
    person_status: typing.Optional[PersonStatusInfo] = None
    position: typing.Optional[PositionInfo] = None
    risk_intelligence: typing.Optional[RiskIntelligenceInfo] = None
    shares: typing.Optional[SharesInfo] = None
    status: typing.Optional[StatusInfo] = None
    translated_name: typing.Optional[TranslatedNameInfo] = None
    weak_identifier: typing.Optional[WeakIdentifierInfo] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.both_identifier_types import BothIdentifierTypes
from ...generated_types.types.country import Country
from ...generated_types.types.entities import Entities
from .profile_enum import ProfileEnum


class ResolutionBody(pydantic_v1.BaseModel):
    name: typing.Optional[typing.List[str]] = pydantic_v1.Field(default=None)
    """
    Entity name
    """

    identifier: typing.Optional[typing.List[BothIdentifierTypes]] = pydantic_v1.Field(default=None)
    """
    Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.
    """

    country: typing.Optional[typing.List[Country]] = pydantic_v1.Field(default=None)
    """
    Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)
    """

    address: typing.Optional[typing.List[str]] = pydantic_v1.Field(default=None)
    """
    Entity address
    """

    date_of_birth: typing.Optional[typing.List[str]] = pydantic_v1.Field(default=None)
    """
    Entity date of birth
    """

    contact: typing.Optional[typing.List[str]] = pydantic_v1.Field(default=None)
    """
    Entity contact
    """

    type: typing.Optional[typing.List[Entities]] = pydantic_v1.Field(default=None)
    """
    [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.
    """

    profile: typing.Optional[ProfileEnum] = pydantic_v1.Field(default=None)
    """
    Profile can be used to switch between search algorithms. The default profile `corporate` is optimized for accurate entity attribute matching and is ideal for business verification and matching entities with corporate data. The `suppliers` profile is optimized for matching entities with extensive trade data. Ideal for supply chain and trade-related use cases.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

IdentifierType = typing.Union[
    typing.Literal[
        "cn_tianyancha_company_id",
        "cn_tianyancha_human_id",
        "cn_unified_social_credit_code",
        "cn_registration_number",
        "cn_organization_code",
        "cn_qichacha_internal_id",
        "cn_hk_cr_number",
        "cn_hk_filing_number",
        "bh_cr_number",
        "bh_cr_number_and_branch",
        "jo_internal_id",
        "jo_national_institution_number",
        "jo_institution_number",
        "malta_company_number",
        "malta_national_id",
        "malta_accountancy_registration_id",
        "uk_company_number",
        "uk_firm_reference_number",
        "uk_person_number",
        "mx_rfc_person",
        "mx_curp",
        "mx_rfc_company",
        "mx_office_fme",
        "ru_inn",
        "ru_ogrn",
        "ven_saren_internal_employer_number",
        "bra_cnpj",
        "ven_rif",
        "ven_cedula_number",
        "mdv_registration_number",
        "ind_director_id_number",
        "ind_permanent_account_number",
        "ind_corporate_id_number",
        "kaz_tin",
        "kaz_bin",
        "kaz_state_reg_num",
        "kaz_okpo_num",
        "kaz_identifier",
        "rks_registration_number",
        "grc_gemi_number",
        "ven_rnc_number",
        "usa_va_reg_id",
        "usa_ny_reg_id",
        "usa_fei_number",
        "ukr_moj_id",
        "mus_reg_id",
        "uk_title_number",
        "mng_reg_number",
        "mne_reg_number",
        "kgz_reg_number",
        "cn_importexport_code",
        "prk_entity_id",
        "prk_registration_id",
        "yugoslav_master_citizen_num",
        "bfa_entity_id",
        "afghan_tin_number",
        "afg_business_license",
        "mdg_nif_number",
        "mdg_stat_number",
        "mdg_rcs_number",
        "lva_personal_code",
        "iban",
        "lva_reg_number",
        "twn_unified_number",
        "lux_rcs_number",
        "mkd_embs_number",
        "mkd_embs_branch_number",
        "mda_idno_number",
        "ury_ruc_number",
        "che_uid_number",
        "slv_commercial_reg_number",
        "slv_uid_number",
        "mato_grosso_legal_id",
        "lei",
        "vat",
        "ita_fiscal_code",
        "ita_vat_number",
        "rio_de_janeiro_legal_id",
        "isl_tin_number",
        "lao_enterprise_number",
        "svk_ico_number",
        "khm_tin_number",
        "cze_ico_number",
        "irn_national_id",
        "ecu_ruc_number",
        "grc_vat_number",
        "can_corporation_number",
        "can_cra_program_account_number",
        "mne_pib",
        "chl_cedula_number",
        "bra_case_number",
        "ind_llpin",
        "ind_fcrn",
        "bmu_registration_number",
        "bra_cpf_number",
        "irn_national_number",
        "chn_customs_registration_code",
        "lie_public_reg_no",
        "cn_tax_identification_number",
        "mac_registration_no",
        "siger_internal_id",
        "lbn_national_id",
        "int_maritime_org_id",
        "ru_ship_register_id",
        "sayari_internal_identifier",
        "aruba_chamber_of_commerce_id",
        "imo_no",
        "vgb_company_number",
        "yem_coc_registration_number",
        "mys_id_card_no",
        "mys_company_reg_no",
        "pry_ruc_number",
        "pry_cedula_number",
        "pry_old_ruc_number",
        "rccm_no",
        "bra_rg_number",
        "ninea_no",
        "alb_tax_id",
        "alb_registration_number",
        "arg_igj_number",
        "arg_dni_number",
        "arg_cuit_number",
        "arg_cuil_number",
        "arg_cdi_number",
        "bih_mbs_number",
        "bih_jib_number",
        "bih_customs_number",
        "usa_puerto_rico_register_number",
        "dji_rcs_number",
        "cri_cedula_number",
        "moz_id_number",
        "moz_nuit_person",
        "moz_nuit_co",
        "moz_passport",
        "moz_dire_no",
        "moz_voter_no",
        "moz_nuel_no",
        "irq_voter_id",
        "jor_voter_card_no",
        "jor_id_no",
        "per_ruc_no",
        "ecu_company_id",
        "usa_ny_dos_id",
        "bfa_rccm_number",
        "fl_prop_folio",
        "usa_fl_pb_pcn",
        "qat_qfc_number",
        "grc_afm_number",
        "vnm_enterprise_code",
        "tha_registration_no",
        "vnm_citizenship_no",
        "vnm_person_id_no",
        "arg_lib_civica_number",
        "arg_lib_enrolamiento_number",
        "nzl_nzbn",
        "nzl_co_no",
        "usa_ofac_sdn_number",
        "swift_bic_code",
        "duns_number",
        "bitcoin_address",
        "litecoin_address",
        "rus_micex_code",
        "tx_prop_id",
        "tx_pacs_id",
        "tx_land_id",
        "png_ipa_reg_id",
        "gtm_nit_number",
        "gtm_cui_number",
        "dom_rnc",
        "qat_cr_number",
        "isin",
        "usa_fl_document_no",
        "usa_fl_fic_name_reg_no",
        "bra_servidor_portal",
        "bra_codigo_da_unidade_organizacional",
        "usa_sec_cik_number",
        "hr_mbs",
        "hr_oib",
        "slv_numero_identificacion_tributaria",
        "slv_mcas",
        "aus_company_number",
        "aus_business_number",
        "deu_registernummer",
        "usa_irs_ein",
        "hong_kong_case_number",
        "irn_coc_internal_id",
        "pan_folio_ficha_id",
        "pan_cedula_number",
        "xxx_cedar_rose_uid",
        "xxx_rccm",
        "vnm_dichvuthongtin_internal_id",
        "xxx_intel_internal_id",
        "usa_washington_state_ubi",
        "nga_registration_number",
        "ng_check_internal_id",
        "nyc_bbl",
        "nyc_crfn",
        "tx_corp_file_num",
        "tx_tax_id",
        "usa_ga_business_id",
        "phl_pse_id",
        "phl_sec_id",
        "phl_tin",
        "afg_passport",
        "irn_passport",
        "kwt_passport",
        "lby_passport",
        "pak_passport",
        "rus_passport",
        "tun_passport",
        "yem_passport",
        "un_sanction_prn",
        "eu_sanction_rn",
        "ca_lp_file_num",
        "tx_entity_filing_num",
        "usa_social_security_number",
        "usa_wy_party_id",
        "usa_wy_filing_id",
        "usa_wy_internal_filing_id",
        "usa_wy_filing_num",
        "usa_or_regno",
        "usa_nv_corpno",
        "usa_nv_bizid",
        "prk_internal_trade_id",
        "lso_corpreg_id",
        "uzb_tin_number",
        "ca_corporate_id_num",
        "gbr_hm_treasury_sanction_group_id",
        "gbr_ipo_trademark_reg_no",
        "usa_ga_control_no",
        "hnd_coc_company_registration_number",
        "mne_property_uid",
        "phl_bnn",
        "rou_company_registration_code",
        "rou_identity_card",
        "rou_personal_id_number",
        "cub_cod",
        "fl_prop_folio_dade",
        "per_dni_no",
        "per_carne_de_extranjeria",
        "fra_siren",
        "fra_siret",
        "fra_rna",
        "fro_reg_num",
        "cze_file_number",
        "esp_borme_reg_id",
        "mex_tm_no",
        "mus_file_no",
        "col_nit_no",
        "jpn_corporate_no",
        "usa_cgac_agency_code",
        "usa_govt_agency_id",
        "usa_govt_office_id",
        "col_cedula_no",
        "col_secop_no",
        "jordan_company_no",
        "dnk_cvr",
        "dnk_production_unit_no",
        "dnk_entity",
        "nor_org_no",
        "swe_org_no",
        "usa_co_reg_no",
        "usa_ia_corp_no",
        "sgp_unqiue_entity_number",
        "usa_ak_entity_no",
        "usa_oh_charter_num",
        "tur_istanbul_coc_reg_no",
        "tur_mersis_number",
        "che_ch_id_number",
        "are_difc_reg_no",
        "idn_tax_id",
        "usa_vt_biz_id",
        "usa_wv_reg_id",
        "usa_ms_biz_id",
        "usa_id_control_no",
        "usa_id_party_id",
        "irn_coc_internal_id_cardno",
        "usa_az_corp_reg_entity_num",
        "usa_ok_filing_no",
        "usa_tn_control_no",
        "usa_tn_party_id",
        "usa_ks_biz_id",
        "usa_hi_corporate_registry_id",
        "usa_hi_corporate_registry_person_id",
        "pol_krs_number",
        "pol_regon_number",
        "pol_nip_number",
        "pol_rejestr_person_id",
        "arm_vat_no",
        "arm_enterprise_code",
        "arm_registration_no",
        "usa_me_corp_id",
        "cyp_reg_no",
        "usa_nd_control_id",
        "usa_mi_corp_id",
        "usa_mi_corp_id_old",
        "usa_hi_trade_name_cert",
        "usa_dc_entity_no",
        "usa_va_old_reg_id",
        "usa_consolidated_screening_list_synthetic_id",
        "usa_ar_filing_no",
        "usa_nc_internal_id",
        "usa_ne_acct_no",
        "usa_ne_agent_id",
        "usa_nm_business_no",
        "usa_nm_license_id",
        "usa_dc_file_no",
        "usa_ri_fei_no",
        "internal_md5",
        "usa_mo_corp_id",
        "usa_wi_dfi_id",
        "geo_identification_code",
        "geo_personal_number",
        "geo_state_registration_number",
        "mac_raem_case_url_id",
        "rou_company_tin",
        "usa_md_dpt_tax",
        "usa_sd_corp_id",
        "hnd_tegucigalpa_matricula",
        "usa_fl_property_id",
        "usa_fl_property_mp_id",
        "usa_fl_property_state_par_id",
        "mex_cluni",
        "usa_pa_corporate_registry_id",
        "pak_ind_ntn",
        "pak_co_ntn",
        "mex_rnie",
        "pak_ngo_reg_no",
        "iraqi_stock_exchange_symbol",
        "bgr_uic",
        "bgr_egn_hashed",
        "south_africa_enterprise_number",
        "south_africa_passport_number",
        "nld_kvk_number",
        "nld_kvk_branch_number",
        "usa_central_registration_depository_number",
        "usa_sec_file_number_bd",
        "chn_cnipa_tm",
        "cyp_passport",
        "cyp_infocredit_entity_id",
        "cyp_id_card",
        "cyp_ssn",
        "aut_firmenbuch_no",
        "swe_per_id_no",
        "chn_shanghai_stock_exchange_company_code",
        "usa_sec_file_number_ia",
        "chn_shenzen_sec_code",
        "usa_ct_business_id",
        "ecu_cedula_number",
        "hkg_stock_code",
        "can_bc_company_registration_id",
        "can_bc_extraprovincial_registration_id",
        "can_ipo_trademark_application_no",
        "ltu_company_registration_code",
        "usa_sec_private_fund",
        "lva_insolvency_proceeding_id",
        "chn_cninfo_internal_shareholder_id",
        "euid",
        "fin_business_id",
        "est_business_reg_code",
        "usa_cusip_number",
        "svn_co_reg_no",
        "isr_company_number",
        "isr_association_number",
        "isr_pbc_number",
        "rks_business_number",
        "rks_fiscal_number",
        "jpn_edinet_code",
        "bel_enterprise_number",
        "bel_establishment_number",
        "aus_afs_licence_number",
        "aus_afs_rep_number",
        "aus_adv_number",
        "aus_credit_licence_number",
        "aus_credit_rep_number",
        "dart_cik",
        "krx_ticker_code",
        "kor_corporate_registration_number",
        "kor_business_tin",
        "est_personal_id",
        "bze_bicar_reg_no",
        "usa_il_chicago_account_number",
        "cym_co_no",
        "ggy_corporate_reg_number",
        "jey_corporate_reg_number",
        "cok_corp_reg_corpid",
        "cok_corp_reg_corpofficerid",
        "cok_corp_reg_number",
        "mex_open_contracts_internal_id",
        "gib_corp_reg_number",
        "dom_onapi_num",
        "rus_tourist_obj",
        "bill_of_lading",
        "twn_factory_registration_number",
        "twn_factory_establishment_permit_case_number",
        "ken_personal_id",
        "col_dian_numero_formulario",
        "vut_corp_reg_number",
        "geo_legal_code",
        "mmr_corp_id",
        "mmr_reg_no",
        "mmr_prior_reg_no",
        "mmr_officer_id",
        "arm_passport_number",
        "ukr_reg_num",
        "validatis_number",
        "are_dubai_land_case_no",
        "ago_matricula_number",
        "ago_nif_number",
        "mmr_personal_id_no",
        "blr_registration_number",
        "aer_free_zone_license",
        "aer_free_zone_reg_no",
        "can_nl_corporate_registry",
        "can_data_axle_hash",
        "svn_ajpes_zapst_number",
        "usa_corpwatch_id",
        "usa_de_file_number",
        "imn_company_number",
        "svn_vat_number",
        "xxx_acuris_id",
        "pak_egm_id",
        "usa_de_registered_agent_id",
        "icij_offshore_internal_id",
        "icij_offshore_node_id",
        "ecu_branch_id",
        "pse_registration_id",
        "srb_mb_number",
        "srb_pib_number",
        "srb_branch_id",
        "rus_bik_code",
        "panadata_internal_id",
        "smr_economic_operator_code",
        "usa_ct_internal_id",
        "alei",
        "can_ns_corporate_registry",
        "som_ubi",
        "bhs_tin",
        "ita_rea_number",
        "mex_denue_clee",
        "ihs_owner_code",
        "aus_consolidated_sanctions_reference",
        "che_seco_sanction_number",
        "gbr_vat_no",
        "gbr_company_number",
        "esp_nif",
        "usa_sam_uei_number",
        "usa_usvi_corp_number",
        "nic_numero_unico",
        "cri_cedula_juridica",
        "cri_cedula_citizen_person",
        "cri_cedula_foreign_person",
        "bol_matricula",
        "bol_old_matricula",
        "nga_crp_reg_internal_id",
        "nga_registration_sn",
        "nga_nin",
        "nga_drivers",
        "nga_tax_id",
        "dma_business_registry_internal_id",
        "cage",
        "atg_business_registry_internal_id",
        "lca_business_registry_internal_id",
        "prt_trust_number",
        "prt_vat_number",
        "mar_passport",
        "cod_passport",
        "prk_passport",
        "sgp_passport",
        "chn_passport",
        "omn_passport",
        "caf_passport",
        "ssd_passport",
        "tto_biz_number",
        "tur_tax_id",
        "bmu_registrar_of_companies_number",
        "cod_rccm_number",
        "cod_rccm_ohada_number",
        "are_reg_auth_number",
        "prk_shipment_id",
        "ecu_importer_id",
        "ecu_exporter_id",
        "ecu_shipment_ref_no",
        "xxx_crb_monitor_entity_id",
        "xxx_edi_global_issuer_id",
        "xxx_edi_global_security_id",
        "chl_import_export_control_id",
        "chl_import_manifest_number",
        "chl_importer_exporter_id",
        "arg_import_export_id",
        "arg_partial_cuit",
        "pan_importer_exporter_id",
        "pan_declaration_number",
        "mex_shipment_number",
        "cri_exp",
        "cri_imp",
        "cri_op_no",
        "col_nur",
        "npl_co_reg_no",
        "usa_sam_exclusions_number",
        "usa_npi_number",
        "usa_upin_number",
        "can_bc_registration_number",
        "can_business_number",
        "chn_csrc_no",
        "usa_mn_master_id",
        "usa_mn_filing_number",
        "usa_in_biz_no",
        "lloyds_internal_vessel_id",
        "lloyds_internal_entity_id",
        "usa_nh_business_id",
        "gbr_uk_sanctions_id",
        "montana_sos_internal_entity_id",
        "usa_mass_sos_company_id",
        "chn_resident_id_number",
        "usa_nj_business_id",
        "utah_corporate_registry_internal_entity_number",
        "usa_la_sos_id",
        "usa_al_sos_id",
        "usa_sc_corp_id",
        "usa_ky_org_no",
        "usa_il_file_number",
        "idn_transaction_number",
        "idn_ubo_owner_id",
        "swe_tax_number",
        "panadata_internal_sid",
        "pry_tax_id",
        "pry_shipment_number",
        "tokyo_stock_exchange_no",
        "swe_fi_id",
        "deu_bafin_id",
        "global_trade_internal_shipment_id",
        "china_vessel_inspection_registration",
        "china_vessel_registration",
        "ccs_registration",
        "usa_oti_id",
        "stock_ticker",
        "can_mb_registry",
        "quebec_enterprise_number",
        "ontario_corporation_number",
        "saskatchewan_registry_number",
        "alberta_corporation_number",
        "bra_shipment_number",
        "ukr_sanctions_nazk_company_internal_id",
        "ukr_sanctions_nazk_person_internal_id",
        "ethereum_address",
        "dash_address",
        "zcash_address",
        "usa_uspto_serial_number",
        "usa_uspto_reg_no",
        "usa_uspto_foreign_application_no",
        "usa_uspto_foreign_reg_no",
        "wipo_intl_reg_no",
        "wipo_intl_ref_no",
        "gbr_charity_no",
        "gbr_trustee_id",
        "gbr_go_no",
        "irl_registration_no",
        "irl_rcn",
        "blz_bccar_reg_no",
        "chn_customs_registration_no",
        "isl_vat_num",
        "can_tm_registration_no",
        "aze_tin_number",
        "tjk_tin_number",
        "tjk_ein_number",
        "mco_rci_number",
        "mco_nis",
        "on_business_id_number",
        "syria_commercial_register_number",
        "cn_hurd_internal_company_id",
        "hun_tax_number",
        "hun_company_register_number",
        "hun_person_tax_id",
        "mw_tpin_tax_reg_num",
        "lr_tin",
        "cm_nui_tax_reg_num",
        "eu_fsd_id",
        "cn_hk_br_number",
        "bwa_uin_reg_number",
        "can_nrd",
        "usa_irs_giin",
        "can_nrd_individual_id",
        "jam_company_id",
        "esp_internal_employee_number",
        "nzl_russia_sanctions_uid",
        "can_new_brunswick_reference_number",
        "fra_asset_freeze_id",
        "srb_beneficial_owner_id",
        "can_pe_registration_number",
        "bel_fsma_name_hash",
        "cuw_reg_number",
        "rus_declaration_number",
        "ru_trade_internal_shipment_id",
        "tur_shipment_id",
        "bd_bin",
        "aer_ner_number",
        "bel_national_register_no",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

GetProjectEntitiesAcceptHeader = typing.Union[
    typing.Literal["application/json", "text/csv", "application/vnd.ms-excel"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...generated_types.types.risk import Risk
from .risk_data import RiskData

EntityRisk = typing.Dict[Risk, RiskData]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .resolution_persisted_response_fields import ResolutionPersistedResponseFields
from .resolution_persisted_result import ResolutionPersistedResult


class ResolutionPersistedResponse(pydantic_v1.BaseModel):
    """
    Examples
    --------
    from sayari import (
        Identifier,
        MatchExplanation,
        MatchStrength,
        ResolutionPersistedResponse,
        ResolutionPersistedResponseFields,
        ResolutionPersistedResult,
    )

    ResolutionPersistedResponse(
        fields=ResolutionPersistedResponseFields(
            name=["victoria beckham limited"],
            custom_name="Victoria Beckham",
        ),
        data=[
            ResolutionPersistedResult(
                saved_entity_id="xG8wYP",
                profile="corporate",
                score=167.28214,
                entity_id="mGq1lpuqKssNWTjIokuPeA",
                label="VICTORIA BECKHAM LIMITED",
                type="company",
                identifiers=[
                    Identifier(
                        type="uk_company_number",
                        value="06517802",
                        label="Uk Company Number",
                    ),
                    Identifier(
                        type="ca_corporate_id_num",
                        value="04781466",
                        label="Ca Corporate Id Num",
                    ),
                    Identifier(
                        type="unknown",
                        value="6517802",
                        label="Unknown",
                    ),
                ],
                psa_id=4655744767230.0,
                addresses=[
                    "202 HAMMERSMITH ROAD , LONDON , , UNITED KINGDOM , W6 7DN , GB",
                    "Unit 33, Ransomes Dock Business Centre, 35-37 Parkgate Road, London SW11 4NP",
                    "SAUNDERS BUILDING, 202 HAMMERSMITH ROAD, HAMMERSMITH, LONDON",
                    "Ransome's Dock, 35-37 Parkgate Road, London, SW11 4NP",
                    "Hammersmith Road, London, W6 7DN",
                    "202 HAMMERSMITH ROAD UNITED KINGDOM",
                    "202 HAMMERSMITH ROAD BRITISH ISLES",
                ],
                countries=["GBR", "USA", "VNM"],
                sources=[
                    "2b618f1996252fe537a6d998ae14c9b2",
                    "2b788dbdf9194ed5a5c309386a6516b1",
                    "a447a7b622c4ead6e1caf94983dc2337",
                    "ecdfb3f2ecc8c3797e77d5795a8066ef",
                    "e5de7b52cc88ef4cd1a10e201bdf46ee",
                    "2a4fe9a14e332c8f9ded1f8a457c2b89",
                    "ecdfb3f2ecc8c3797e77d5795a8066ef",
                    "4ea8bac1bed868e1510ffd21842e9551",
                ],
                typed_matched_queries=["name|0", "looseName|0"],
                matched_queries=["name"],
                highlight={
                    "name": [
                        "<em>VICTORIA</em> <em>BECKHAM</em> LIMITED",
                        "<em>BECKHAM</em> VENTURES LIMITED",
                    ]
                },
                explanation={
                    "name": [
                        MatchExplanation(
                            matched="<em>VICTORIA</em> <em>BECKHAM</em> LIMITED",
                            uploaded="victoria beckham limited",
                            name_custom_tf_idf_score=0.5416772821044128,
                            high_quality_match_name=True,
                            is_deletion_recommended=False,
                        ),
                        MatchExplanation(
                            matched="<em>BECKHAM</em> VENTURES LIMITED",
                            uploaded="victoria beckham limited",
                            name_custom_tf_idf_score=0.5,
                            high_quality_match_name=False,
                            is_deletion_recommended=False,
                        ),
                    ]
                },
                match_strength=MatchStrength(
                    value="weak",
                ),
            )
        ],
    )
    """

    fields: ResolutionPersistedResponseFields
    data: typing.List[ResolutionPersistedResult]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..generated_types.types.both_identifier_types import BothIdentifierTypes
from ..generated_types.types.country import Country
from ..generated_types.types.entities import Entities
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_acceptable import NotAcceptable
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_acceptable_response import NotAcceptableResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.profile_enum import ProfileEnum
from .types.resolution_body import ResolutionBody
from .types.resolution_persisted_response import ResolutionPersistedResponse
from .types.resolution_response import ResolutionResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ResolutionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def resolution(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]] = None,
        country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        address: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        contact: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        profile: typing.Optional[ProfileEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        name : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity name

        identifier : typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]]
            Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.

        country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)

        address : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity address

        date_of_birth : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity date of birth

        contact : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity contact

        type : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.

        profile : typing.Optional[ProfileEnum]
            Profile can be used to switch between search algorithms. The default profile `corporate` is optimized for accurate entity attribute matching and is ideal for business verification and matching entities with corporate data. The `suppliers` profile is optimized for matching entities with extensive trade data. Ideal for supply chain and trade-related use cases.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resolution.resolution(
            name="victoria beckham limited",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
                "name": name,
                "identifier": identifier,
                "country": country,
                "address": address,
                "date_of_birth": date_of_birth,
                "contact": contact,
                "type": type,
                "profile": profile,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ResolutionResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 406:
                raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def resolution_post(
        self,
        *,
        request: ResolutionBody,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        request : ResolutionBody

        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        from sayari import ResolutionBody
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resolution.resolution_post(
            limit=1,
            request=ResolutionBody(
                name=["victoria beckham limited"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="POST",
            params={"limit": limit, "offset": offset},
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ResolutionResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 406:
                raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def resolution_persisted(
        self,
        project_id: str,
        *,
        request: ResolutionBody,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionPersistedResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> The persisted resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the resolution endpoint, except it also stores matched entities into user's project.

        Parameters
        ----------
        project_id : str
            Unique identifier of the project

        request : ResolutionBody

        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionPersistedResponse

        Examples
        --------
        from sayari import ResolutionBody
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resolution.resolution_persisted(
            project_id="V03eYM",
            limit=1,
            request=ResolutionBody(
                name=["victoria beckham limited"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/resolution/persisted/{jsonable_encoder(project_id)}",
            method="POST",
            params={"limit": limit, "offset": offset},
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ResolutionPersistedResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 406:
                raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncResolutionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def resolution(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]] = None,
        country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        address: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        contact: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        profile: typing.Optional[ProfileEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        name : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity name

        identifier : typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]]
            Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.

        country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)

        address : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity address

        date_of_birth : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity date of birth

        contact : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity contact

        type : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.

        profile : typing.Optional[ProfileEnum]
            Profile can be used to switch between search algorithms. The default profile `corporate` is optimized for accurate entity attribute matching and is ideal for business verification and matching entities with corporate data. The `suppliers` profile is optimized for matching entities with extensive trade data. Ideal for supply chain and trade-related use cases.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        import asyncio

        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.resolution.resolution(
                name="victoria beckham limited",
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
                "name": name,
                "identifier": identifier,
                "country": country,
                "address": address,
                "date_of_birth": date_of_birth,
                "contact": contact,
                "type": type,
                "profile": profile,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ResolutionResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 406:
                raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def resolution_post(
        self,
        *,
        request: ResolutionBody,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        request : ResolutionBody

        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        import asyncio

        from sayari import ResolutionBody
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.resolution.resolution_post(
                limit=1,
                request=ResolutionBody(
                    name=["victoria beckham limited"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="POST",
            params={"limit": limit, "offset": offset},
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ResolutionResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 406:
                raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def resolution_persisted(
        self,
        project_id: str,
        *,
        request: ResolutionBody,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionPersistedResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> The persisted resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the resolution endpoint, except it also stores matched entities into user's project.

        Parameters
        ----------
        project_id : str
            Unique identifier of the project

        request : ResolutionBody

        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionPersistedResponse

        Examples
        --------
        import asyncio

        from sayari import ResolutionBody
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.resolution.resolution_persisted(
                project_id="V03eYM",
                limit=1,
                request=ResolutionBody(
                    name=["victoria beckham limited"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/resolution/persisted/{jsonable_encoder(project_id)}",
            method="POST",
            params={"limit": limit, "offset": offset},
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ResolutionPersistedResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 406:
                raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..generated_types.types.country import Country
from ..generated_types.types.relationships import Relationships
from ..generated_types.types.risk import Risk
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.entity_summary_response import EntitySummaryResponse
from .types.get_entity_response import GetEntityResponse


class EntityClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_entity(
        self,
        id: str,
        *,
        attributes_additional_information_next: typing.Optional[str] = None,
        attributes_additional_information_prev: typing.Optional[str] = None,
        attributes_additional_information_limit: typing.Optional[int] = None,
        attributes_address_next: typing.Optional[str] = None,
        attributes_address_prev: typing.Optional[str] = None,
        attributes_address_limit: typing.Optional[int] = None,
        attributes_business_purpose_next: typing.Optional[str] = None,
        attributes_business_purpose_prev: typing.Optional[str] = None,
        attributes_business_purpose_limit: typing.Optional[int] = None,
        attributes_company_type_next: typing.Optional[str] = None,
        attributes_company_type_prev: typing.Optional[str] = None,
        attributes_company_type_limit: typing.Optional[int] = None,
        attributes_country_next: typing.Optional[str] = None,
        attributes_country_prev: typing.Optional[str] = None,
        attributes_country_limit: typing.Optional[int] = None,
        attributes_identifier_next: typing.Optional[str] = None,
        attributes_identifier_prev: typing.Optional[str] = None,
        attributes_identifier_limit: typing.Optional[int] = None,
        attributes_name_next: typing.Optional[str] = None,
        attributes_name_prev: typing.Optional[str] = None,
        attributes_name_limit: typing.Optional[int] = None,
        attributes_status_next: typing.Optional[str] = None,
        attributes_status_prev: typing.Optional[str] = None,
        attributes_status_limit: typing.Optional[int] = None,
        relationships_next: typing.Optional[str] = None,
        relationships_prev: typing.Optional[str] = None,
        relationships_limit: typing.Optional[int] = None,
        relationships_type: typing.Optional[Relationships] = None,
        relationships_sort: typing.Optional[str] = None,
        relationships_start_date: typing.Optional[dt.date] = None,
        relationships_end_date: typing.Optional[dt.date] = None,
        relationships_min_shares: typing.Optional[int] = None,
        relationships_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_state: typing.Optional[str] = None,
        relationships_arrival_city: typing.Optional[str] = None,
        relationships_departure_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_departure_state: typing.Optional[str] = None,
        relationships_departure_city: typing.Optional[str] = None,
        relationships_partner_name: typing.Optional[str] = None,
        relationships_partner_risk: typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]] = None,
        relationships_hs_code: typing.Optional[str] = None,
        possibly_same_as_next: typing.Optional[str] = None,
        possibly_same_as_prev: typing.Optional[str] = None,
        possibly_same_as_limit: typing.Optional[int] = None,
        referenced_by_next: typing.Optional[str] = None,
        referenced_by_prev: typing.Optional[str] = None,
        referenced_by_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEntityResponse:
        """
        Retrieve an entity from the database based on the ID

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        attributes_additional_information_next : typing.Optional[str]
            The pagination token for the next page of attribute `additional_information`.

        attributes_additional_information_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `additional_information`.

        attributes_additional_information_limit : typing.Optional[int]
            Limit total values returned for attribute `additional_information`. Defaults to 100.

        attributes_address_next : typing.Optional[str]
            The pagination token for the next page of attribute `address`.

        attributes_address_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `address`.

        attributes_address_limit : typing.Optional[int]
            Limit total values returned for attribute `address`. Defaults to 100.

        attributes_business_purpose_next : typing.Optional[str]
            The pagination token for the next page of attribute `business_purpose`.

        attributes_business_purpose_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `business_purpose`.

        attributes_business_purpose_limit : typing.Optional[int]
            Limit total values returned for attribute `business_purpose`. Defaults to 100.

        attributes_company_type_next : typing.Optional[str]
            The pagination token for the next page of attribute `company_type`.

        attributes_company_type_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `company_type`.

        attributes_company_type_limit : typing.Optional[int]
            Limit total values returned for attribute `company_type`. Defaults to 100.

        attributes_country_next : typing.Optional[str]
            The pagination token for the next page of attribute `country`.

        attributes_country_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `country`.

        attributes_country_limit : typing.Optional[int]
            Limit total values returned for attribute `country`. Defaults to 100.

        attributes_identifier_next : typing.Optional[str]
            The pagination token for the next page of attribute `identifier`.

        attributes_identifier_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `identifier`.

        attributes_identifier_limit : typing.Optional[int]
            Limit total values returned for attribute `identifier`. Defaults to 100.

        attributes_name_next : typing.Optional[str]
            The pagination token for the next page of attribute `name`.

        attributes_name_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `name`.

        attributes_name_limit : typing.Optional[int]
            Limit total values returned for attribute `name`. Defaults to 100.

        attributes_status_next : typing.Optional[str]
            The pagination token for the next page of attribute `status`.

        attributes_status_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `status`.

        attributes_status_limit : typing.Optional[int]
            Limit total values returned for attribute `status`. Defaults to 100.

        relationships_next : typing.Optional[str]
            The pagination token for the next page of relationship results

        relationships_prev : typing.Optional[str]
            The pagination token for the previous page of relationship results

        relationships_limit : typing.Optional[int]
            Limit total relationship values. Defaults to 50.

        relationships_type : typing.Optional[Relationships]
            Filter relationships to [relationship type](/sayari-library/ontology/relationships), e.g. director_of or has_shareholder

        relationships_sort : typing.Optional[str]
            Sorts relationships by As Of date or Shareholder percentage, e.g. date or -shares

        relationships_start_date : typing.Optional[dt.date]
            Filters relationships to after a date

        relationships_end_date : typing.Optional[dt.date]
            Filters relationships to before a date

        relationships_min_shares : typing.Optional[int]
            Filters relationships to greater than or equal to a Shareholder percentage

        relationships_country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filters relationships to a list of [countries](/sayari-library/ontology/enumerated-types#country)

        relationships_arrival_country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filters shipment relationships to a list of arrival [countries](/sayari-library/ontology/enumerated-types#country)

        relationships_arrival_state : typing.Optional[str]
            Filters shipment relationships to an arrival state

        relationships_arrival_city : typing.Optional[str]
            Filters shipment relationships to an arrival city

        relationships_departure_country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filters shipment relationships to a list of departure [countries](/sayari-library/ontology/enumerated-types#country)

        relationships_departure_state : typing.Optional[str]
            Filters shipment relationships to a departure state

        relationships_departure_city : typing.Optional[str]
            Filters shipment relationships to a departure city

        relationships_partner_name : typing.Optional[str]
            Filters shipment relationships to a trade partner name

        relationships_partner_risk : typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]]
            Filters shipment relationships to a trade partner [risk tag](/sayari-library/ontology/enumerated-types#tag)

        relationships_hs_code : typing.Optional[str]
            Filters shipment relationships to an HS code

        possibly_same_as_next : typing.Optional[str]
            The pagination token for the next page of possibly same entities.

        possibly_same_as_prev : typing.Optional[str]
            The pagination token for the previous page of possibly same entities.

        possibly_same_as_limit : typing.Optional[int]
            Limit total possibly same as entities. Defaults to 100.

        referenced_by_next : typing.Optional[str]
            The pagination token for the next page of the entity's referencing records

        referenced_by_prev : typing.Optional[str]
            The pagination token for the previous page of the entity's referencing records

        referenced_by_limit : typing.Optional[int]
            Limit totals values returned for entity's referencing records. Defaults to 100.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEntityResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.entity.get_entity(
            id="mGq1lpuqKssNWTjIokuPeA",
            attributes_name_limit=1,
            attributes_address_limit=1,
            attributes_country_limit=1,
            attributes_additional_information_limit=1,
            attributes_business_purpose_limit=1,
            attributes_company_type_limit=1,
            attributes_identifier_limit=1,
            attributes_status_limit=1,
            relationships_limit=1,
            possibly_same_as_limit=1,
            referenced_by_limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/entity/{jsonable_encoder(id)}",
            method="GET",
            params={
                "attributes.additional_information.next": attributes_additional_information_next,
                "attributes.additional_information.prev": attributes_additional_information_prev,
                "attributes.additional_information.limit": attributes_additional_information_limit,
                "attributes.address.next": attributes_address_next,
                "attributes.address.prev": attributes_address_prev,
                "attributes.address.limit": attributes_address_limit,
                "attributes.business_purpose.next": attributes_business_purpose_next,
                "attributes.business_purpose.prev": attributes_business_purpose_prev,
                "attributes.business_purpose.limit": attributes_business_purpose_limit,
                "attributes.company_type.next": attributes_company_type_next,
                "attributes.company_type.prev": attributes_company_type_prev,
                "attributes.company_type.limit": attributes_company_type_limit,
                "attributes.country.next": attributes_country_next,
                "attributes.country.prev": attributes_country_prev,
                "attributes.country.limit": attributes_country_limit,
                "attributes.identifier.next": attributes_identifier_next,
                "attributes.identifier.prev": attributes_identifier_prev,
                "attributes.identifier.limit": attributes_identifier_limit,
                "attributes.name.next": attributes_name_next,
                "attributes.name.prev": attributes_name_prev,
                "attributes.name.limit": attributes_name_limit,
                "attributes.status.next": attributes_status_next,
                "attributes.status.prev": attributes_status_prev,
                "attributes.status.limit": attributes_status_limit,
                "relationships.next": relationships_next,
                "relationships.prev": relationships_prev,
                "relationships.limit": relationships_limit,
                "relationships.type": relationships_type,
                "relationships.sort": relationships_sort,
                "relationships.startDate": str(relationships_start_date)
                if relationships_start_date is not None
                else None,
                "relationships.endDate": str(relationships_end_date) if relationships_end_date is not None else None,
                "relationships.minShares": relationships_min_shares,
                "relationships.country": relationships_country,
                "relationships.arrivalCountry": relationships_arrival_country,
                "relationships.arrivalState": relationships_arrival_state,
                "relationships.arrivalCity": relationships_arrival_city,
                "relationships.departureCountry": relationships_departure_country,
                "relationships.departureState": relationships_departure_state,
                "relationships.departureCity": relationships_departure_city,
                "relationships.partnerName": relationships_partner_name,
                "relationships.partnerRisk": relationships_partner_risk,
                "relationships.hsCode": relationships_hs_code,
                "possibly_same_as.next": possibly_same_as_next,
                "possibly_same_as.prev": possibly_same_as_prev,
                "possibly_same_as.limit": possibly_same_as_limit,
                "referenced_by.next": referenced_by_next,
                "referenced_by.prev": referenced_by_prev,
                "referenced_by.limit": referenced_by_limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(GetEntityResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def entity_summary(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EntitySummaryResponse:
        """
        The Entity Summary endpoint returns a smaller entity payload

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntitySummaryResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.entity.entity_summary(
            id="mGq1lpuqKssNWTjIokuPeA",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/entity_summary/{jsonable_encoder(id)}", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(EntitySummaryResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncEntityClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_entity(
        self,
        id: str,
        *,
        attributes_additional_information_next: typing.Optional[str] = None,
        attributes_additional_information_prev: typing.Optional[str] = None,
        attributes_additional_information_limit: typing.Optional[int] = None,
        attributes_address_next: typing.Optional[str] = None,
        attributes_address_prev: typing.Optional[str] = None,
        attributes_address_limit: typing.Optional[int] = None,
        attributes_business_purpose_next: typing.Optional[str] = None,
        attributes_business_purpose_prev: typing.Optional[str] = None,
        attributes_business_purpose_limit: typing.Optional[int] = None,
        attributes_company_type_next: typing.Optional[str] = None,
        attributes_company_type_prev: typing.Optional[str] = None,
        attributes_company_type_limit: typing.Optional[int] = None,
        attributes_country_next: typing.Optional[str] = None,
        attributes_country_prev: typing.Optional[str] = None,
        attributes_country_limit: typing.Optional[int] = None,
        attributes_identifier_next: typing.Optional[str] = None,
        attributes_identifier_prev: typing.Optional[str] = None,
        attributes_identifier_limit: typing.Optional[int] = None,
        attributes_name_next: typing.Optional[str] = None,
        attributes_name_prev: typing.Optional[str] = None,
        attributes_name_limit: typing.Optional[int] = None,
        attributes_status_next: typing.Optional[str] = None,
        attributes_status_prev: typing.Optional[str] = None,
        attributes_status_limit: typing.Optional[int] = None,
        relationships_next: typing.Optional[str] = None,
        relationships_prev: typing.Optional[str] = None,
        relationships_limit: typing.Optional[int] = None,
        relationships_type: typing.Optional[Relationships] = None,
        relationships_sort: typing.Optional[str] = None,
        relationships_start_date: typing.Optional[dt.date] = None,
        relationships_end_date: typing.Optional[dt.date] = None,
        relationships_min_shares: typing.Optional[int] = None,
        relationships_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_state: typing.Optional[str] = None,
        relationships_arrival_city: typing.Optional[str] = None,
        relationships_departure_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_departure_state: typing.Optional[str] = None,
        relationships_departure_city: typing.Optional[str] = None,
        relationships_partner_name: typing.Optional[str] = None,
        relationships_partner_risk: typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]] = None,
        relationships_hs_code: typing.Optional[str] = None,
        possibly_same_as_next: typing.Optional[str] = None,
        possibly_same_as_prev: typing.Optional[str] = None,
        possibly_same_as_limit: typing.Optional[int] = None,
        referenced_by_next: typing.Optional[str] = None,
        referenced_by_prev: typing.Optional[str] = None,
        referenced_by_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEntityResponse:
        """
        Retrieve an entity from the database based on the ID

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        attributes_additional_information_next : typing.Optional[str]
            The pagination token for the next page of attribute `additional_information`.

        attributes_additional_information_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `additional_information`.

        attributes_additional_information_limit : typing.Optional[int]
            Limit total values returned for attribute `additional_information`. Defaults to 100.

        attributes_address_next : typing.Optional[str]
            The pagination token for the next page of attribute `address`.

        attributes_address_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `address`.

        attributes_address_limit : typing.Optional[int]
            Limit total values returned for attribute `address`. Defaults to 100.

        attributes_business_purpose_next : typing.Optional[str]
            The pagination token for the next page of attribute `business_purpose`.

        attributes_business_purpose_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `business_purpose`.

        attributes_business_purpose_limit : typing.Optional[int]
            Limit total values returned for attribute `business_purpose`. Defaults to 100.

        attributes_company_type_next : typing.Optional[str]
            The pagination token for the next page of attribute `company_type`.

        attributes_company_type_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `company_type`.

        attributes_company_type_limit : typing.Optional[int]
            Limit total values returned for attribute `company_type`. Defaults to 100.

        attributes_country_next : typing.Optional[str]
            The pagination token for the next page of attribute `country`.

        attributes_country_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `country`.

        attributes_country_limit : typing.Optional[int]
            Limit total values returned for attribute `country`. Defaults to 100.

        attributes_identifier_next : typing.Optional[str]
            The pagination token for the next page of attribute `identifier`.

        attributes_identifier_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `identifier`.

        attributes_identifier_limit : typing.Optional[int]
            Limit total values returned for attribute `identifier`. Defaults to 100.

        attributes_name_next : typing.Optional[str]
            The pagination token for the next page of attribute `name`.

        attributes_name_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `name`.

        attributes_name_limit : typing.Optional[int]
            Limit total values returned for attribute `name`. Defaults to 100.

        attributes_status_next : typing.Optional[str]
            The pagination token for the next page of attribute `status`.

        attributes_status_prev : typing.Optional[str]
            The pagination token for the previous page of attribute `status`.

        attributes_status_limit : typing.Optional[int]
            Limit total values returned for attribute `status`. Defaults to 100.

        relationships_next : typing.Optional[str]
            The pagination token for the next page of relationship results

        relationships_prev : typing.Optional[str]
            The pagination token for the previous page of relationship results

        relationships_limit : typing.Optional[int]
            Limit total relationship values. Defaults to 50.

        relationships_type : typing.Optional[Relationships]
            Filter relationships to [relationship type](/sayari-library/ontology/relationships), e.g. director_of or has_shareholder

        relationships_sort : typing.Optional[str]
            Sorts relationships by As Of date or Shareholder percentage, e.g. date or -shares

        relationships_start_date : typing.Optional[dt.date]
            Filters relationships to after a date

        relationships_end_date : typing.Optional[dt.date]
            Filters relationships to before a date

        relationships_min_shares : typing.Optional[int]
            Filters relationships to greater than or equal to a Shareholder percentage

        relationships_country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filters relationships to a list of [countries](/sayari-library/ontology/enumerated-types#country)

        relationships_arrival_country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filters shipment relationships to a list of arrival [countries](/sayari-library/ontology/enumerated-types#country)

        relationships_arrival_state : typing.Optional[str]
            Filters shipment relationships to an arrival state

        relationships_arrival_city : typing.Optional[str]
            Filters shipment relationships to an arrival city

        relationships_departure_country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filters shipment relationships to a list of departure [countries](/sayari-library/ontology/enumerated-types#country)

        relationships_departure_state : typing.Optional[str]
            Filters shipment relationships to a departure state

        relationships_departure_city : typing.Optional[str]
            Filters shipment relationships to a departure city

        relationships_partner_name : typing.Optional[str]
            Filters shipment relationships to a trade partner name

        relationships_partner_risk : typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]]
            Filters shipment relationships to a trade partner [risk tag](/sayari-library/ontology/enumerated-types#tag)

        relationships_hs_code : typing.Optional[str]
            Filters shipment relationships to an HS code

        possibly_same_as_next : typing.Optional[str]
            The pagination token for the next page of possibly same entities.

        possibly_same_as_prev : typing.Optional[str]
            The pagination token for the previous page of possibly same entities.

        possibly_same_as_limit : typing.Optional[int]
            Limit total possibly same as entities. Defaults to 100.

        referenced_by_next : typing.Optional[str]
            The pagination token for the next page of the entity's referencing records

        referenced_by_prev : typing.Optional[str]
            The pagination token for the previous page of the entity's referencing records

        referenced_by_limit : typing.Optional[int]
            Limit totals values returned for entity's referencing records. Defaults to 100.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEntityResponse

        Examples
        --------
        import asyncio

        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.entity.get_entity(
                id="mGq1lpuqKssNWTjIokuPeA",
                attributes_name_limit=1,
                attributes_address_limit=1,
                attributes_country_limit=1,
                attributes_additional_information_limit=1,
                attributes_business_purpose_limit=1,
                attributes_company_type_limit=1,
                attributes_identifier_limit=1,
                attributes_status_limit=1,
                relationships_limit=1,
                possibly_same_as_limit=1,
                referenced_by_limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/entity/{jsonable_encoder(id)}",
            method="GET",
            params={
                "attributes.additional_information.next": attributes_additional_information_next,
                "attributes.additional_information.prev": attributes_additional_information_prev,
                "attributes.additional_information.limit": attributes_additional_information_limit,
                "attributes.address.next": attributes_address_next,
                "attributes.address.prev": attributes_address_prev,
                "attributes.address.limit": attributes_address_limit,
                "attributes.business_purpose.next": attributes_business_purpose_next,
                "attributes.business_purpose.prev": attributes_business_purpose_prev,
                "attributes.business_purpose.limit": attributes_business_purpose_limit,
                "attributes.company_type.next": attributes_company_type_next,
                "attributes.company_type.prev": attributes_company_type_prev,
                "attributes.company_type.limit": attributes_company_type_limit,
                "attributes.country.next": attributes_country_next,
                "attributes.country.prev": attributes_country_prev,
                "attributes.country.limit": attributes_country_limit,
                "attributes.identifier.next": attributes_identifier_next,
                "attributes.identifier.prev": attributes_identifier_prev,
                "attributes.identifier.limit": attributes_identifier_limit,
                "attributes.name.next": attributes_name_next,
                "attributes.name.prev": attributes_name_prev,
                "attributes.name.limit": attributes_name_limit,
                "attributes.status.next": attributes_status_next,
                "attributes.status.prev": attributes_status_prev,
                "attributes.status.limit": attributes_status_limit,
                "relationships.next": relationships_next,
                "relationships.prev": relationships_prev,
                "relationships.limit": relationships_limit,
                "relationships.type": relationships_type,
                "relationships.sort": relationships_sort,
                "relationships.startDate": str(relationships_start_date)
                if relationships_start_date is not None
                else None,
                "relationships.endDate": str(relationships_end_date) if relationships_end_date is not None else None,
                "relationships.minShares": relationships_min_shares,
                "relationships.country": relationships_country,
                "relationships.arrivalCountry": relationships_arrival_country,
                "relationships.arrivalState": relationships_arrival_state,
                "relationships.arrivalCity": relationships_arrival_city,
                "relationships.departureCountry": relationships_departure_country,
                "relationships.departureState": relationships_departure_state,
                "relationships.departureCity": relationships_departure_city,
                "relationships.partnerName": relationships_partner_name,
                "relationships.partnerRisk": relationships_partner_risk,
                "relationships.hsCode": relationships_hs_code,
                "possibly_same_as.next": possibly_same_as_next,
                "possibly_same_as.prev": possibly_same_as_prev,
                "possibly_same_as.limit": possibly_same_as_limit,
                "referenced_by.next": referenced_by_next,
                "referenced_by.prev": referenced_by_prev,
                "referenced_by.limit": referenced_by_limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(GetEntityResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def entity_summary(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EntitySummaryResponse:
        """
        The Entity Summary endpoint returns a smaller entity payload

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntitySummaryResponse

        Examples
        --------
        import asyncio

        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.entity.entity_summary(
                id="mGq1lpuqKssNWTjIokuPeA",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/entity_summary/{jsonable_encoder(id)}", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(EntitySummaryResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx

from .attributes.client import AsyncAttributesClient, AttributesClient
from .auth.client import AsyncAuthClient, AuthClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.oauth_token_provider import OAuthTokenProvider
from .entity.client import AsyncEntityClient, EntityClient
from .environment import SayariEnvironment
from .info.client import AsyncInfoClient, InfoClient
from .metadata.client import AsyncMetadataClient, MetadataClient
from .notifications.client import AsyncNotificationsClient, NotificationsClient
from .project.client import AsyncProjectClient, ProjectClient
from .record.client import AsyncRecordClient, RecordClient
from .resolution.client import AsyncResolutionClient, ResolutionClient
from .resource.client import AsyncResourceClient, ResourceClient
from .search.client import AsyncSearchClient, SearchClient
from .source.client import AsyncSourceClient, SourceClient
from .supply_chain.client import AsyncSupplyChainClient, SupplyChainClient
from .trade.client import AsyncTradeClient, TradeClient
from .traversal.client import AsyncTraversalClient, TraversalClient


class BaseClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SayariEnvironment
        The environment to use for requests from the client. from .environment import SayariEnvironment



        Defaults to SayariEnvironment.PRODUCTION



    client_id : str
    client_secret : str
    _token_getter_override : typing.Optional[typing.Callable[[], str]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from sayari.client import Sayari

    client = Sayari(
        client_id="YOUR_CLIENT_ID",
        client_secret="YOUR_CLIENT_SECRET",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SayariEnvironment = SayariEnvironment.PRODUCTION,
        client_id: str,
        client_secret: str,
        _token_getter_override: typing.Optional[typing.Callable[[], str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        oauth_token_provider = OAuthTokenProvider(
            client_id=client_id,
            client_secret=client_secret,
            client_wrapper=SyncClientWrapper(
                base_url=_get_base_url(base_url=base_url, environment=environment),
                httpx_client=httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.Client(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
            ),
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=_token_getter_override if _token_getter_override is not None else oauth_token_provider.get_token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.attributes = AttributesClient(client_wrapper=self._client_wrapper)
        self.auth = AuthClient(client_wrapper=self._client_wrapper)
        self.entity = EntityClient(client_wrapper=self._client_wrapper)
        self.info = InfoClient(client_wrapper=self._client_wrapper)
        self.metadata = MetadataClient(client_wrapper=self._client_wrapper)
        self.notifications = NotificationsClient(client_wrapper=self._client_wrapper)
        self.project = ProjectClient(client_wrapper=self._client_wrapper)
        self.record = RecordClient(client_wrapper=self._client_wrapper)
        self.resolution = ResolutionClient(client_wrapper=self._client_wrapper)
        self.resource = ResourceClient(client_wrapper=self._client_wrapper)
        self.search = SearchClient(client_wrapper=self._client_wrapper)
        self.source = SourceClient(client_wrapper=self._client_wrapper)
        self.supply_chain = SupplyChainClient(client_wrapper=self._client_wrapper)
        self.trade = TradeClient(client_wrapper=self._client_wrapper)
        self.traversal = TraversalClient(client_wrapper=self._client_wrapper)


class AsyncBaseClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SayariEnvironment
        The environment to use for requests from the client. from .environment import SayariEnvironment



        Defaults to SayariEnvironment.PRODUCTION



    client_id : str
    client_secret : str
    _token_getter_override : typing.Optional[typing.Callable[[], str]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from sayari.client import AsyncSayari

    client = AsyncSayari(
        client_id="YOUR_CLIENT_ID",
        client_secret="YOUR_CLIENT_SECRET",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SayariEnvironment = SayariEnvironment.PRODUCTION,
        client_id: str,
        client_secret: str,
        _token_getter_override: typing.Optional[typing.Callable[[], str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        oauth_token_provider = OAuthTokenProvider(
            client_id=client_id,
            client_secret=client_secret,
            client_wrapper=SyncClientWrapper(
                base_url=_get_base_url(base_url=base_url, environment=environment),
                httpx_client=httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.Client(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
            ),
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=_token_getter_override if _token_getter_override is not None else oauth_token_provider.get_token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.attributes = AsyncAttributesClient(client_wrapper=self._client_wrapper)
        self.auth = AsyncAuthClient(client_wrapper=self._client_wrapper)
        self.entity = AsyncEntityClient(client_wrapper=self._client_wrapper)
        self.info = AsyncInfoClient(client_wrapper=self._client_wrapper)
        self.metadata = AsyncMetadataClient(client_wrapper=self._client_wrapper)
        self.notifications = AsyncNotificationsClient(client_wrapper=self._client_wrapper)
        self.project = AsyncProjectClient(client_wrapper=self._client_wrapper)
        self.record = AsyncRecordClient(client_wrapper=self._client_wrapper)
        self.resolution = AsyncResolutionClient(client_wrapper=self._client_wrapper)
        self.resource = AsyncResourceClient(client_wrapper=self._client_wrapper)
        self.search = AsyncSearchClient(client_wrapper=self._client_wrapper)
        self.source = AsyncSourceClient(client_wrapper=self._client_wrapper)
        self.supply_chain = AsyncSupplyChainClient(client_wrapper=self._client_wrapper)
        self.trade = AsyncTradeClient(client_wrapper=self._client_wrapper)
        self.traversal = AsyncTraversalClient(client_wrapper=self._client_wrapper)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: SayariEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.country import Country


class RecordDetails(pydantic_v1.BaseModel):
    id: str = pydantic_v1.Field()
    """
    The unique identifier for a record in the database
    """

    label: str
    source: str = pydantic_v1.Field()
    """
    The unique identifier for a source in the database
    """

    publication_date: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    The date the record was published
    """

    acquisition_date: str = pydantic_v1.Field()
    """
    The date Sayari acquired this record
    """

    references_count: int = pydantic_v1.Field()
    """
    Number of times this record is referenced
    """

    record_url: str = pydantic_v1.Field()
    """
    The url to download the record from Sayari
    """

    source_url: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    The url to access to original source
    """

    document_urls: typing.Optional[typing.List[str]] = None
    matches: typing.Optional[typing.Dict[str, typing.List[str]]] = None
    country: typing.Optional[Country] = None
    page: typing.Optional[float] = None
    page_count: typing.Optional[float] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .project import Project


class CreateProjectResponse(pydantic_v1.BaseModel):
    """
    Examples
    --------
    from sayari import CreateProjectResponse, Project, ProjectCounts

    CreateProjectResponse(
        data=Project(
            id="YVMBRg",
            label="Project Alpha",
            archived=False,
            created="2024-03-15 20:31:06.08855+00",
            updated="2024-03-15 20:31:06.08855+00",
            counts=ProjectCounts(
                entity=0,
                graph=0,
                record=0,
                search=0,
            ),
        ),
    )
    """

    data: Project

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .bucket_agg import BucketAgg
from .hs_code_agg import HsCodeAgg
from .tier_count_agg import TierCountAgg


class ProjectEntitiesAggs(pydantic_v1.BaseModel):
    """
    Aggregation buckets for entities in a project.
    """

    hit_count: typing.Optional[typing.List[BucketAgg]] = None
    country: typing.Optional[typing.List[BucketAgg]] = None
    upstream_country: typing.Optional[typing.List[BucketAgg]] = None
    upstream_country_tiers: typing.Optional[TierCountAgg] = None
    risk: typing.Optional[typing.List[BucketAgg]] = None
    upstream_risk: typing.Optional[typing.List[BucketAgg]] = None
    upstream_risk_tiers: typing.Optional[TierCountAgg] = None
    source: typing.Optional[typing.List[BucketAgg]] = None
    business_purpose: typing.Optional[typing.List[BucketAgg]] = None
    tag_ids: typing.Optional[typing.List[BucketAgg]] = None
    case_statuses: typing.Optional[typing.List[BucketAgg]] = None
    shipment_counts: typing.Optional[typing.List[BucketAgg]] = None
    shipped_hs_codes: typing.Optional[HsCodeAgg] = None
    received_hs_codes: typing.Optional[HsCodeAgg] = None
    match_results: typing.Optional[typing.List[BucketAgg]] = None
    location: typing.Optional[typing.List[BucketAgg]] = None
    source_type: typing.Optional[typing.List[BucketAgg]] = None
    region: typing.Optional[typing.List[BucketAgg]] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .user_info import UserInfo


class MetadataResponse(pydantic_v1.BaseModel):
    """
    Examples
    --------
    from sayari import MetadataResponse, UserInfo

    MetadataResponse(
        version="v2.1.0-4841-g41af84af8",
        master_release="reiko_20240712182628",
        watchlist_release="watchlist_reiko_20240718024049",
        user=UserInfo(
            id="abcdefghijklmnopqrstuvwxyz@clients",
            group_display_names="",
            roles="",
        ),
    )
    """

    version: str = pydantic_v1.Field()
    """
    Currently deployed version of the application.
    """

    master_release: str = pydantic_v1.Field()
    """
    Currently deployed main data release.
    """

    watchlist_release: str = pydantic_v1.Field()
    """
    Currently deployed watchlist release.
    """

    user: UserInfo

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.delete_resource_response import DeleteResourceResponse
from .types.resource_type import ResourceType
from .types.save_entity_request import SaveEntityRequest
from .types.save_entity_response import SaveEntityResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ResourceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def save_entity(
        self, *, request: SaveEntityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> SaveEntityResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Save an entity to a project.

        Parameters
        ----------
        request : SaveEntityRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SaveEntityResponse

        Examples
        --------
        from sayari import SaveEntityRequest
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resource.save_entity(
            request=SaveEntityRequest(
                type="entity",
                project="GNJbkG",
                entity_id="Zk0qOaM2SSYg_ZhsljykMQ",
                custom_fields={"properties": {"custom_name": "Victoria Beckham"}},
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/resource/entity", method="POST", json=request, request_options=request_options, omit=OMIT
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(SaveEntityResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_resource(
        self, type: ResourceType, resource_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteResourceResponse:
        """
        Deletes an existing saved resource from a project.

        Parameters
        ----------
        type : ResourceType

        resource_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteResourceResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resource.delete_resource(
            type="entity",
            resource_id="YWmNKV",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/resource/{jsonable_encoder(type)}/{jsonable_encoder(resource_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(DeleteResourceResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncResourceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def save_entity(
        self, *, request: SaveEntityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> SaveEntityResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Save an entity to a project.

        Parameters
        ----------
        request : SaveEntityRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SaveEntityResponse

        Examples
        --------
        import asyncio

        from sayari import SaveEntityRequest
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.resource.save_entity(
                request=SaveEntityRequest(
                    type="entity",
                    project="GNJbkG",
                    entity_id="Zk0qOaM2SSYg_ZhsljykMQ",
                    custom_fields={"properties": {"custom_name": "Victoria Beckham"}},
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/resource/entity", method="POST", json=request, request_options=request_options, omit=OMIT
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(SaveEntityResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_resource(
        self, type: ResourceType, resource_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteResourceResponse:
        """
        Deletes an existing saved resource from a project.

        Parameters
        ----------
        type : ResourceType

        resource_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteResourceResponse

        Examples
        --------
        import asyncio

        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.resource.delete_resource(
                type="entity",
                resource_id="YWmNKV",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/resource/{jsonable_encoder(type)}/{jsonable_encoder(resource_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(DeleteResourceResponse, _response.json())  # type: ignore
            if _response.status_code == 400:
                raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
            if _response.status_code == 401:
                raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
            if _response.status_code == 404:
                raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json())  # type: ignore
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
            if _response.status_code == 500:
                raise InternalServerError(
                    pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.risk import Risk
from ...shared_types.types.risk_value import RiskValue
from .notification_additional_information import NotificationAdditionalInformation
from .notification_type import NotificationType


class Notification(pydantic_v1.BaseModel):
    type: NotificationType = pydantic_v1.Field()
    """
    The type of notification, currently limited to 'risk'
    """

    field: Risk = pydantic_v1.Field()
    """
    The field that the notification is for
    """

    values: typing.List[RiskValue] = pydantic_v1.Field()
    """
    The previous values of the field
    """

    date: str = pydantic_v1.Field()
    """
    The date the notification was created
    """

    additional_information: typing.Optional[NotificationAdditionalInformation] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from ...core.api_error import ApiError
from ..types.connection_error_response import ConnectionErrorResponse


class ConnectionError(ApiError):
    def __init__(self, body: ConnectionErrorResponse):
        super().__init__(status_code=520, body=body)



# This file was auto-generated by Fern from our API Definition.

import typing

CountryContext = typing.Union[
    typing.Literal[
        "citizenship",
        "incorporation",
        "residence",
        "nationality",
        "address",
        "vessel_flag",
        "domicile",
        "shipment_departure",
        "shipment_arrival",
        "shipment_transit",
        "activity_in",
        "mentioned_in",
        "product_origin",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

from ...base_types.types.paginated_response import PaginatedResponse
from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class EntityRelationships(PaginatedResponse):
    """
    All relationships the entity is a part of.
    """

    next: typing.Optional[typing.Any] = None
    data: typing.List[RelationshipData]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


from .relationship_data import RelationshipData  # noqa: E402

EntityRelationships.update_forward_refs()



# This file was auto-generated by Fern from our API Definition.

from .get_record_response import GetRecordResponse
from .record_references import RecordReferences

__all__ = ["GetRecordResponse", "RecordReferences"]
